from __future__ import absolute_import

from django import forms
from django.contrib import admin
from django.contrib.admin.models import LogEntry
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.core.urlresolvers import clear_url_caches
from six.moves import reload_module

from robustredirects.models import Redirect
from robustredirects.utils import ignored_url_paths


class RedirectModelForm(forms.ModelForm):
    class Meta:
        model = Redirect
        fields = ('site', 'from_url', 'to_url', 'http_status', 'status', 'is_partial',
                  'uses_regex',)

    def clean(self):
        cleaned_data = super(RedirectModelForm, self).clean()

        if cleaned_data['is_partial'] and cleaned_data['uses_regex']:
            raise ValidationError('Redirect cannot be partial and also a regular expression.')

        ignored_urls = ignored_url_paths()
        from_url = cleaned_data['from_url']
        ignored_prefix = next((u for u in ignored_urls if from_url.startswith(u)), None)
        if ignored_prefix is not None:
            raise ValidationError('Redirect matches ignored path: %s' % ignored_prefix)


class RedirectAdmin(admin.ModelAdmin):
    form = RedirectModelForm
    list_display = ['from_url', 'to_url', 'is_partial', 'uses_regex', 'site', 'status', 'edit']
    readonly_fields = ["edit"]
    change_list_template = "change_form.html"

    def save_model(self, request, object, form, change):
        from robustredirects import dynamic_urls
        instance = form.save()
        # for sites that are not in debug mode reload
        # the dynamic urls, i'm not sure if this is the
        # best way though
        reload_module(dynamic_urls)
        clear_url_caches()
        return instance

    def logs(self, **filters):
        content_type = ContentType.objects.get_for_model(self.model)
        return LogEntry.objects.filter(content_type=content_type, **filters).order_by('-pk')

    def edit(self, obj):
        log = self.logs(object_id=obj.id).first()
        return "{} ({})".format(log.action_time.strftime("%Y-%m-%d %H:%M %Z"), log.user)

    def history(self):
        return [{
            "description": log,
            "id": log.object_id,
            "user": log.user,
            "time": log.action_time.strftime("%Y-%m-%d %H:%M %Z"),
        } for log in self.logs()[:20]]

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context["history"] = self.history
        return super(RedirectAdmin, self).changelist_view(request, extra_context)


admin.site.register(Redirect, RedirectAdmin)



from __future__ import absolute_import, print_function

from django.core.exceptions import ObjectDoesNotExist
from django.core.management.base import BaseCommand, CommandError
from robustredirects.models import Redirect
from django.contrib.redirects.models import Redirect as DjangoRedirect
from django.db import transaction


class Command(BaseCommand):
    can_import_settings = True

    def handle(self, *args, **options):
        with transaction.commit_on_success():
            count = 0
            for redirect in DjangoRedirect.objects.all():
                try:
                    Redirect.objects.get(from_url=redirect.old_path)
                    print("Redirect from {} already exists...skipping".format(redirect.old_path))
                except ObjectDoesNotExist:
                    new_redirect = Redirect(from_url=redirect.old_path, to_url=redirect.new_path, site=redirect.site,
                                            http_status=301)

                    new_redirect.save()
                    count += 1

        print("Copied {} redirects into robust redirects.".format(count))


from __future__ import absolute_import

from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from django.core.urlresolvers import resolve, Resolver404
from django.db.models import Q
from django.http import HttpResponsePermanentRedirect, HttpResponseRedirect, HttpResponseGone
from robustredirects.models import Redirect
from robustredirects.utils import ignored_url_paths, replace_partial_url

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:  # Django < 1.10
    # backwards compatibility for MIDDLEWARE_CLASSES
    MiddlewareMixin = object


class RedirectMiddleware(MiddlewareMixin):
    """
        Process the redirect patterns from redirects.dynamic_urls.
    """

    def try_resolve(self, path, request):
        urlconf = 'robustredirects.dynamic_urls'
        redirect, args, kwargs = resolve(path, urlconf=urlconf)
        return redirect(request, **kwargs)

    def process_response(self, request, response):
        if response.status_code != 404:
            # No need to check for a redirect for non-404 responses.
            return response

        if request.path.startswith(ignored_url_paths()):
            # Stop checking ignored urls
            return response

        path = request.get_full_path()
        try:
            return self.try_resolve(path, request)
        except Resolver404:
            pass

        # Try again by changing the slash
        try:
            # if not, check if adding/removing the trailing slash helps
            if path.endswith('/'):
                new_path = path[:-1]
            else:
                new_path = path + '/'

            return self.try_resolve(new_path, request)
        except Resolver404:
            pass

        current_site = get_current_site(request)

        # No regex redirect was found try a simple replace
        if path.startswith('/'):
            no_leading_slash_path = path[1:]
            leading_slash_path = path
        else:
            no_leading_slash_path = path
            leading_slash_path = '/' + path

        # Try looking for an exact match
        redirect = Redirect.objects.filter(
            Q(from_url__iexact=no_leading_slash_path) | Q(from_url__iexact=leading_slash_path),
            status=1,
            site=current_site).order_by('-pk').first()

        if redirect:
            if redirect.to_url == '':
                return HttpResponseGone()

            # Do a replace on the url and do a redirect
            path = replace_partial_url(path, redirect.from_url, redirect.to_url)

            if redirect.http_status == 301:
                return HttpResponsePermanentRedirect(path)
            elif redirect.http_status == 302:
                return HttpResponseRedirect(path)

        # Try looking for a true partial
        db_filters = {
            'status': 1,
            'site': current_site,
            'is_partial': True
        }

        redirects = Redirect.objects.filter(**db_filters)

        for redirect in redirects:
            if redirect.from_url in path:
                if redirect.to_url == '':
                    return HttpResponseGone()

                # Do a replace on the url and do a redirect
                path = replace_partial_url(path, redirect.from_url, redirect.to_url)

                if redirect.http_status == 301:
                    return HttpResponsePermanentRedirect(path)
                elif redirect.http_status == 302:
                    return HttpResponseRedirect(path)

        return response



from __future__ import absolute_import

from django.conf.urls import url
from django.conf import settings
from robustredirects.models import Redirect
from robustredirects import views


def ignored_url_paths():
    return getattr(settings, "ROBUST_REDIRECTS_IGNORED_PREFIXES", ())


def group_arguments(seq, group=254):
    """
        group the list into lists of 254 items each.

        This is due to argument restrictions in python.
        http://docs.djangoproject.com/en/dev/topics/http/urls/#patterns
    """
    return (seq[pos:pos + group] for pos in range(0, len(seq), group))


def get_redirect_patterns():
    """
        Gets the redirect patterns out of the database
        and assigns them to the django patterns object.
    """
    site_id = settings.SITE_ID
    url_patterns = []
    url_list = []
    db_filters = {
        'status': 1,
        'site': site_id,
        'is_partial': False,
        'uses_regex': True
    }

    redirects = Redirect.objects.filter(**db_filters)
    for redirect in redirects:
        extra = {}
        pattern = r'^%s$' % redirect.from_url

        extra.update({'url': '%s' % redirect.to_url})

        if redirect.http_status == 302:
            extra.update({'permanent': False})
            url_list.append(url(pattern, views.redirect_to, extra))
        else:
            url_list.append(url(pattern, views.redirect_to, extra))

    arg_groups = list(group_arguments(url_list))
    for args in arg_groups:
        url_patterns += list(args)

    return url_patterns


def replace_partial_url(starting_url, replace_from, replace_to):
    """
    Simple helper at this point. Maybe useful if we want to try supporting regex in the future.
    """
    new_url = starting_url.replace(replace_from, replace_to)

    if not new_url.startswith('/'):
        new_url = '/' + new_url

    return new_url



# view patterns
from __future__ import absolute_import

import logging
from django.http import HttpResponsePermanentRedirect, HttpResponseRedirect, HttpResponseGone

logger = logging.getLogger('django.request')


def redirect_to(request, url, permanent=True, query_string=False, **kwargs):
    """
    Redirect to a given URL.

    The given url may contain dict-style string formatting, which will be
    interpolated against the params in the URL.  For example, to redirect from
    ``/foo/<id>/`` to ``/bar/<id>/``, you could use the following URLconf::

        urlpatterns = patterns('',
            ('^foo/(?P<id>\d+)/$', 'django.views.generic.simple.redirect_to', {'url' : '/bar/%(id)s/'}),
        )

    If the given url is ``None``, a HttpResponseGone (410) will be issued.

    If the ``permanent`` argument is False, then the response will have a 302
    HTTP status code. Otherwise, the status code will be 301.

    If the ``query_string`` argument is True, then the GET query string
    from the request is appended to the URL.

    """
    args = request.META.get('QUERY_STRING', '')

    if url not in (None, ''):
        if kwargs:
            url = url % kwargs

        if args and query_string:
            url = "%s?%s" % (url, args)

        klass = permanent and HttpResponsePermanentRedirect or HttpResponseRedirect
        return klass(url)
    else:
        logger.warning('Gone: %s' % request.path,
                    extra={
                        'status_code': 410,
                        'request': request
                    })
        return HttpResponseGone()



from __future__ import absolute_import

from six.moves import reload_module

from django.conf import settings
from django.http import HttpResponseNotFound
from django.test import TestCase
from django.test.client import RequestFactory
from django.contrib.sites.shortcuts import get_current_site
from django.core.urlresolvers import clear_url_caches

from robustredirects.middleware import RedirectMiddleware
from robustredirects.models import Redirect


class TestRedirectMiddleWare(TestCase):
    def setUp(self):
        super(TestRedirectMiddleWare, self).setUp()
        # Every test needs access to the request factory.
        self.factory = RequestFactory()
        # tests should start with an empty url cache
        clear_url_caches()
        settings.ROBUST_REDIRECTS_IGNORED_URL_PATHS = None

    @staticmethod
    def run_redirect(request):
        from robustredirects import dynamic_urls

        reload_module(dynamic_urls)
        middleware = RedirectMiddleware()
        response = HttpResponseNotFound()
        new_response = middleware.process_response(request, response)
        return new_response

    def test_redirect_request_permanent(self):
        # Create a redirect
        request = self.factory.get('/test/123/')

        redirect = Redirect(from_url=r'test/(?P<pk>\d+)/', to_url=r'somethingelse/(?P<pk>\d+)/',
                            site=get_current_site(request), uses_regex=True)

        redirect.save()
        new_response = self.run_redirect(request)

        self.assertEqual(new_response.status_code, 301)
        assert 'somethingelse' in new_response.serialize_headers()

    def test_redirect_request_gone(self):
        # Create a redirect
        request = self.factory.get('/test/123/')

        redirect = Redirect(from_url=r'test/(?P<pk>\d+)/', to_url='',
                            site=get_current_site(request), uses_regex=True)

        redirect.save()
        new_response = self.run_redirect(request)

        self.assertEqual(new_response.status_code, 410)

    def test_redirect_request_temporary(self):
        # Create a redirect
        request = self.factory.get('/test/123/')

        redirect = Redirect(from_url=r'test/(?P<pk>\d+)/', to_url=r'somethingelse/(?P<pk>\d+)/',
                            site=get_current_site(request), http_status=302, uses_regex=True)

        redirect.save()
        new_response = self.run_redirect(request)

        self.assertEqual(new_response.status_code, 302)
        assert 'somethingelse' in new_response.serialize_headers()

    def test_redirect_request_partial_temporary(self):
        # Create a redirect
        request = self.factory.get('/test/123/')

        redirect = Redirect(from_url='/test/', to_url='/partialtest/', is_partial=True,
                            site=get_current_site(request), http_status=302)

        redirect.save()
        new_response = self.run_redirect(request)

        self.assertEqual(new_response.status_code, 302)
        assert 'partialtest' in new_response.serialize_headers()

    def test_redirect_request_partial_permanent(self):
        # Create a redirect
        request = self.factory.get('/test/123/')

        redirect = Redirect(from_url='/test/', to_url='/partialtest/', is_partial=True,
                            site=get_current_site(request), http_status=301)

        redirect.save()
        new_response = self.run_redirect(request)

        self.assertEqual(new_response.status_code, 301)
        assert 'partialtest' in new_response.serialize_headers()

    def test_redirect_request_two_partial_entries_permanent(self):
        # Create a redirect
        old_route = '/invalidroot/partialtest'
        redirected_route = '/test/partialtest'
        request = self.factory.get(old_route)

        redirect = Redirect(from_url='/invalidroot', to_url=redirected_route, is_partial=True,
                            site=get_current_site(request), http_status=301)

        redirect.save()
        redirect2 = Redirect(from_url=old_route, to_url=redirected_route, is_partial=True,
                             site=get_current_site(request), http_status=301)

        redirect2.save()
        new_response = self.run_redirect(request)

        self.assertEqual(new_response.status_code, 301)
        self.assertEqual(new_response.url, redirected_route)

    def test_redirect_request_partial_gone(self):
        # Create a redirect
        request = self.factory.get('/test/123/')

        redirect = Redirect(from_url='/test/', to_url='', is_partial=True,
                            site=get_current_site(request), http_status=301)

        redirect.save()
        new_response = self.run_redirect(request)

        self.assertEqual(new_response.status_code, 410)

    def test_redirect_request_partial_prepend_slash(self):
        # Create a redirect
        request = self.factory.get('/test/123/')

        redirect = Redirect(from_url='/test/', to_url='partialtest/', is_partial=True,
                            site=get_current_site(request), http_status=302)

        redirect.save()
        new_response = self.run_redirect(request)

        self.assertEqual(new_response.status_code, 302)
        assert '/partialtest/123/' in new_response.serialize_headers()

    def test_redirect_exclusion(self):
        # Create a redirect
        request = self.factory.get('/api/test/123/')

        settings.ROBUST_REDIRECTS_IGNORED_PREFIXES = '/api'

        redirect = Redirect(from_url='/test/', to_url='partialtest/', is_partial=True,
                            site=get_current_site(request), http_status=302)

        redirect.save()
        redirect2 = Redirect(from_url=r'/api/test/(?P<pk>\d+)/', to_url=r'somethingelse/(?P<pk>\d+)/',
                             site=get_current_site(request), http_status=302, uses_regex=True)

        redirect2.save()
        new_response = self.run_redirect(request)

        # no redirect should happen
        self.assertEqual(new_response.status_code, 404)



from __future__ import absolute_import

from django.db import models
from django.conf import settings
from django.contrib.sites.models import Site
from django.utils.translation import ugettext_lazy as _

HTTP_STATUS_CHOICES = (
    (301, _('301 - Permanent Redirect')),
    (302, _('302 - Temporary Redirect')),
)

STATUS_CHOICES = (
    (True, _('Active')),
    (False, _('Inactive')),
)

uses_regex_helptext = _('Check if the From URL uses a regular expression. '
                        'If so, it will be processed in a urlconf.')

from_url_helptext = _('Absolute path, excluding the domain. '
                      'Example: \'/about/\''
                      )

to_url_helptext = _('Absolute or relative url. Examples: '
                    '\'http://www.example.com\', \'/something/\''
                    )

is_partial_helptext = _('The From and To URL are partial. They will be used if they partially match any part of the'
                        ' url. Can not be a regular expression.')


class Redirect(models.Model):
    site = models.ForeignKey(Site, related_name='robust_redirects')

    from_url = models.CharField(_('From URL'), max_length=255, unique=True,
                                db_index=True, help_text=from_url_helptext)

    to_url = models.CharField(_('To URL'), max_length=255,
                              db_index=True, help_text=to_url_helptext, blank=True)

    http_status = models.SmallIntegerField(_('HTTP Status'),
                                           choices=HTTP_STATUS_CHOICES,
                                           default=301)

    status = models.BooleanField(choices=STATUS_CHOICES, default=True)

    is_partial = models.BooleanField(_('Is a partial url'),
                                     default=False,
                                     help_text=is_partial_helptext)

    uses_regex = models.BooleanField(_('Uses Regular Expression'),
                                     default=False,
                                     help_text=uses_regex_helptext)

    create_dt = models.DateTimeField(auto_now_add=True)
    update_dt = models.DateTimeField(auto_now=True)

    class Meta:
        verbose_name = _('redirect')
        verbose_name_plural = _('redirects')
        unique_together = (('site', 'from_url'),)
        ordering = ('-uses_regex',)

    def __unicode__(self):
        return _("Redirect: %(from)s --> %(to)s") % {'from': self.from_url, 'to': self.to_url}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

setup(
    name="django-robust-redirects",
    version="0.13.2",
    url="http://github.com/spothero/django-robust-redirects",
    download_url="http://github.com/spothero/django-robust-redirects/tarball/0.10.0",
    description="A more robust and feature full django redirect package",
    author="SpotHero",
    author_email="pypi@spothero.com",
    packages=["robustredirects"],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
    ],
    install_requires=[
        "django>=1.10.3,<2",
        "future>=0.17.1,<1",
        "six>=1.12.0,<2"
    ],
)
